-- Tests for the HazLoot core functions

-- TODO: In-game testing of opening / closing corpses. Tests work, but it has bugged in practice
-- TODO: Change to use mock items, not BP and gun

require("WoWMock")
require("Core")
require("HSK")

function TestSetup()
  print("*** Setting up tests")
  HazLoot.OnInitialize()
  WM:AddParty()
  print("*** Setup succeeded")
end

function TestTeardown()
  print("*** Tearing down tests")
  print("*** Teardown succeeded")
end

function TestXXX()
  print("*** Testing HazLoot:XXX()")
  WM:CleanUp()
  print("*** HazLoot:XXX() is working as expected.")
end

function TestGetItemId()
  print("*** Testing HazLoot:GetItemId(itemLink)")
  WM:CreateCorpse(WM.THREEGUN)
  assert(41168 == WM.corpse[2], "Expected 2nd slot to have a combat shotgun")
  local tmp = HazLoot:GetItemId(GetLootSlotLink(2))
  assert(41168 == tmp, "Expected itemId to be 41168, not: "..(tmp or "nil"))
  WM:CleanUp()
  print("*** HazLoot:GetItemId(itemLink) is working as expected.")
end

function TestLootThatBag()
  print("*** Testing HazLoot:LootThatBag()")
  assert(0 == #HazLoot.PhatLewt, "Expected PhatLewt to be empty")
  assert(HazLoot.STATE_IDLE == HazLoot.State, "Expected HazLoot to be IDLE")

  print("\n* Testing an empty bag *")
  WM:CreateCorpse(WM.EMPTY)
  HazLoot:LootThatBag()
  assert(0 == #HazLoot.PhatLewt, "Expected PhatLewt to be empty")
  assert(HazLoot.STATE_IDLE == HazLoot.State, "Expected HazLoot to be IDLE")

  print("\n* Testing a money-only bag *")
  WM:CreateCorpse(WM.MONEYONLY)
  assert(0 < #WM.corpse, "Corpse should not be empty")
  HazLoot:LootThatBag()
  WM:CloseCorpse()
  assert(0 == #WM.corpse, "Corpse should be empty")
  assert(0 == #HazLoot.PhatLewt, "Expected PhatLewt to be empty")
  assert(HazLoot.STATE_IDLE == HazLoot.State, "Expected HazLoot to be IDLE")

  print("\n* Testing a corpse with cash and five greens - five in party, one not master loot eligible *")
  WM:CreateCorpse(WM.FIVEGREEN)
  assert(6 == #WM.corpse, "Corpse should have six slots: money + 5 greens")
  HazLoot:LootThatBag()
  WM:CloseCorpse()
  assert(0 == #WM.corpse, "Corpse should be empty")
  assert(0 == HazLoot.LootList[HazLoot.UNCOMMON]["Wexler"], "Wexler shouldn't have any green loot")
  -- Expected behaviour here is not obvious

  print("\n* Testing a corpse with cash, a green and two epics")
  WM:CreateCorpse(WM.MIXED)
  assert(0 < #WM.corpse, "Corpse should not be empty")
  assert(4 == #WM.corpse, "Corpse should have 4 loot slots")
  assert(not HazLoot:OpenNext(), "Should not have been able to open a bid")
  HazLoot:LootThatBag()
  HazLoot:ShowMeLoot(HazLoot.PhatLewt)
  assert(nil == HazLoot.CurrentItem, "Current Item should be nil")
  assert(HazLoot:OpenNext(), "Should have been able to open a bid")
  assert(HazLoot.STATE_AUCTION_OPEN == HazLoot.State, "Expected HazLoot to be AUCTION_OPEN")
  assert(1 == HazLoot.CurrentItem, "Current Item should be #1")
  assert(4 == HazLoot.PhatLewt[HazLoot.CurrentItem].slot, "Should be slot 4")
  assert(0 < #WM.corpse, "Corpse should not be empty")
  assert(0 ~= #HazLoot.PhatLewt, "PhatLewt should not be empty")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  WM:CloseCorpse()
  assert(HazLoot.STATE_AUCTION_OPEN == HazLoot.State, "Expected HazLoot to be AUCTION_OPEN")
  HazLoot:LootThatBag()
  HazLoot:ShowMeLoot(HazLoot.PhatLewt)
  assert(2 == #WM.corpse, "Corpse should have 2 loot slots")
  assert(1 == HazLoot.CurrentItem, "Should be item #1")
  assert(2 == HazLoot.PhatLewt[HazLoot.CurrentItem].slot, "Should be slot 2")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  assert(HazLoot.STATE_AUCTION_OPEN == HazLoot.State, "Expected HazLoot to be AUCTION_OPEN")
  assert(not HazLoot:Confirm({}), "Expected Confirm() to fail: no candidate")
  assert(HazLoot:LastCall(),"Expected to be able to issue a last call")
  assert(not HazLoot:Confirm({}), "Expected Confirm() to fail: no candidate proposed")
  assert(2 == #WM.corpse, "Corpse should have 2 loot slots")
  assert(1 == HazLoot.CurrentItem, "Should be item #1")
  assert(2 == HazLoot.PhatLewt[HazLoot.CurrentItem].slot, "Should be slot 2")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  assert(HazLoot.STATE_AUCTION_LASTCALL == HazLoot.State, "Expected HazLoot to be AUCTION_LASTCALL")
  WM:CloseCorpse()
  assert(HazLoot.STATE_AUCTION_LASTCALL == HazLoot.State, "Expected HazLoot to be AUCTION_LASTCALL")
  HazLoot:LootThatBag()
  HazLoot:ShowMeLoot(HazLoot.PhatLewt)
  assert(not HazLoot:SetCandidate("", "Avoc Vikong Valeris Vargar"), "Expected to fail on list of invalid winners")
  assert(not HazLoot:GetCandidate().name, "Expected candidate to be nil")
  assert(HazLoot:SetCandidate("", "hazarakhan Groupal avOc vaRGar donkey"), "Expected to succeed on mixed list of valid and invalid winners")
  assert("Groupal" == HazLoot:GetCandidate().name, "Expected candidate to be Groupal")
  assert(HazLoot:Confirm({}), "Expected confirm to succeed: Groupal is candidate")
  assert(2 == #WM.corpse, "Corpse should have 2 loot slots")
  assert(1 == HazLoot.CurrentItem, "Should be item #1")
  assert(2 == HazLoot.PhatLewt[HazLoot.CurrentItem].slot, "Should be slot 2")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  assert(HazLoot.STATE_AUCTION_CONFIRMED == HazLoot.State, "Expected HazLoot to be AUCTION_CONFIRMED, got: "..HazLoot.STATES[HazLoot.State])
  assert(not HazLoot:LastCall(), "Expected lastcall to fail after confirmation")
  assert(HazLoot.STATE_AUCTION_CONFIRMED == HazLoot.State, "Expected HazLoot to be AUCTION_CONFIRMED")
  WM:CloseCorpse()
  assert(HazLoot.STATE_AUCTION_CONFIRMED == HazLoot.State, "Expected HazLoot to be AUCTION_CONFIRMED")
  HazLoot:LootThatBag()
  HazLoot:ShowMeLoot(HazLoot.PhatLewt)
  assert(HazLoot:AwardCurrentItem(info), "Expected loot to go be awarded")
  assert(HazLoot.STATE_SENDING_ITEM == HazLoot.State, "Expected HazLoot to be SENDING_ITEM")
  assert(2 == #WM.corpse, "Corpse should have 2 loot slots")
  assert(1 == HazLoot.CurrentItem, "Should be item #1")
  assert(2 == HazLoot.PhatLewt[HazLoot.CurrentItem].slot, "Should be slot 2")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  assert(HazLoot:AwardCurrentItem(info), "Expected loot to go to Groupal") -- confirm idempotent until drop or loot cleared
  assert(HazLoot.STATE_SENDING_ITEM == HazLoot.State, "Expected HazLoot to be SENDING_ITEM")
  assert(2 == #WM.corpse, "Corpse should have 2 loot slots")
  assert(1 == HazLoot.CurrentItem, "Should be item #1")
  assert(2 == HazLoot.PhatLewt[HazLoot.CurrentItem].slot, "Should be slot 2")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  assert(35 == HazLoot.DropWinner(), "Groupal's new position should be 35")
  HazLoot:LootCleared(2) -- Clear slot 2, since WoW can't do it for us
  assert(0 == #HazLoot:GetCandidate(), "Candidate should now be empty")
  assert(2 == #WM.corpse, "Corpse should have 2 loot slots")
  assert(HazLoot.STATE_LOOTING == HazLoot.State, "Expected HazLoot to be LOOTING")
  assert(not HazLoot.CurrentItem, "Current item should be nil")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  assert(HazLoot:OpenNext(), "Should have been able to open a bid")
  assert(2 == #WM.corpse, "Corpse should have 2 loot slots")
  assert(HazLoot.STATE_AUCTION_OPEN == HazLoot.State, "Expected HazLoot to be AUCTION_OPEN")
  assert(2 == HazLoot.CurrentItem, "Current item should be #2")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  WM:CloseCorpse()
  assert(1 == #WM.corpse, "Corpse should have 1 loot slots")
  assert(HazLoot.STATE_AUCTION_OPEN == HazLoot.State, "Expected HazLoot to be AUCTION_OPEN")
  assert(2 == HazLoot.CurrentItem, "Current item should be #2")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  assert(HazLoot:LastCall(),"Expected to be able to issue a last call")
  assert(HazLoot.STATE_AUCTION_LASTCALL == HazLoot.State, "Expected HazLoot to be AUCTION_LASTCALL")
  assert(HazLoot:LastCall(),"Expected to be able to issue a last call twice")
  assert(HazLoot.STATE_AUCTION_LASTCALL == HazLoot.State, "Expected HazLoot to be AUCTION_LASTCALL")
  assert(HazLoot:OpenToRoll(),"Expected to be able to open item to rolls")
  assert(HazLoot.STATE_ROLL_OPEN == HazLoot.State, "Expected HazLoot to be ROLL_OPEN")
  assert(HazLoot:LastCall(),"Expected to be able to issue a last call")
  assert(HazLoot.STATE_ROLL_LASTCALL == HazLoot.State, "Expected HazLoot to be ROLL_LASTCALL")
  assert(HazLoot:LastCall(),"Expected to be able to issue a last call twice")
  assert(HazLoot.STATE_ROLL_LASTCALL == HazLoot.State, "Expected HazLoot to be ROLL_LASTCALL")
  assert(not HazLoot:Confirm({}), "Expected confirm to fail - must use SetCandidate()")
  assert(HazLoot:SetCandidate({},"Parrot"), "Expected SetCandidate({},\"Parrot\") to succeed")
  assert(HazLoot:Confirm({}), "Expected confirm to succeed")
  assert(HazLoot.STATE_ROLL_CONFIRMED == HazLoot.State, "Expected HazLoot to be ROLL_CONFIRMED")
  assert(HazLoot:AwardCurrentItem(info), "Expected loot to go to Parrot")
  assert(HazLoot.STATE_SENDING_ITEM == HazLoot.State, "Expected HazLoot to be SENDING_ITEM")
  assert(1 == #WM.corpse, "Corpse should have 1 loot slots")
  assert(2 == HazLoot.CurrentItem, "Should be item #2")
  assert(1 == HazLoot.PhatLewt[HazLoot.CurrentItem].slot, "Should be slot 1")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  assert(HazLoot:AwardCurrentItem(info), "Expected loot to go to Parrot (again)") -- confirm idempotent /hl award
  assert(HazLoot.STATE_SENDING_ITEM == HazLoot.State, "Expected HazLoot to be SENDING_ITEM")
  assert(1 == #WM.corpse, "Corpse should have 1 loot slots")
  assert(2 == HazLoot.CurrentItem, "Should be item #2")
  assert(1 == HazLoot.PhatLewt[HazLoot.CurrentItem].slot, "Should be slot 1")
  assert(2 == #HazLoot.PhatLewt, "PhatLewt should have 2 items")
  HazLoot:LootCleared(1) -- Clear slot 2, since WoW can't do it for us
  assert(1 == #WM.corpse, "Corpse should have 1 loot slots")
  assert(not HazLoot.CurrentItem, "Current item should be nil")
  assert(not HazLoot.PhatLewt, "PhatLewt should be nil")
  assert(not HazLoot:OpenNext(), "Should not have been able to open a bid")
  WM:CloseCorpse()

  print("\n* Testing automatic win assignment from Strategy (with cash and one epic)")
  assert(0 == #HazLoot.GetCandidate(), "Candidate table should be empty")
  WM:CreateCorpse(WM.ONEEPIC)
  HazLoot.LootThatBag()
  assert(HazLoot.STATE_IDLE == HazLoot.State, "Expected HazLoot to be IDLE")
  assert(HazLoot:OpenNext(), "Should have been able to open a bid")
  assert(HazLoot.STATE_AUCTION_OPEN == HazLoot.State, "Expected HazLoot to be AUCTION_OPEN")
  assert(HazLoot:LastCall(),"Expected to be able to issue a last call")
  print(HSK:ExportPresentList())
  -- Now, I've got bids from Wexler, Parrot and Groupal
  assert(HazLoot:SetCandidate("","wexler parrot groupal"), "Should have been able to set a winner")
  assert("Parrot" == HazLoot:GetCandidate().name, "Parrot should be the winner")
  assert(not HazLoot:AwardCurrentItem(), "Expected to be unable to award loot without confirmation")
  assert(HazLoot:Confirm(),"Should be able to confirm set winner")
  assert("Parrot" == HazLoot:GetCandidate().name, "Parrot should be the winner")
  assert(HazLoot:AwardCurrentItem(), "Expected to be able to award loot after confirmation")
  assert("Parrot" == HazLoot:GetCandidate().name, "Parrot should be the winner")
  HazLoot:LootCleared(1) -- Clear slot 2, since WoW can't do it for us
  assert(HazLoot:DropWinner(), "Expected to be able to drop the winner in the Strategy")
  assert(0 == #HazLoot.GetCandidate(), "Candidate table should be empty")
  print("<=========")

  print(HSK:ExportPresentList())
  HazLoot:GetStatusInfo(false)

  WM:CleanUp()
  print("*** HazLoot:LootThatBag() is working as expected.")
end

TestSetup()
TestGetItemId()
TestLootThatBag()
TestTeardown()
